@@ -63,14 +63,19 @@
  * Returns: %TRUE if FREERDP headers are new enough
  */
 
-#define FREERDP_CHECK_VERSION(major,minor,release)                                  \
+#define FREERDP_CHECK_VERSION(major,minor,revision)                                  \
 	(FREERDP_VERSION_MAJOR > (major) ||                                       \
 	 (FREERDP_VERSION_MAJOR == (major) && FREERDP_VERSION_MINOR > (minor)) || \
 	 (FREERDP_VERSION_MAJOR == (major) && FREERDP_VERSION_MINOR == (minor) && \
-	  FREERDP_VERSION_RELEASE >= (release)))
+	  FREERDP_VERSION_REVISION >= (revision)))
 
-/* Constants to workaround FreeRDP issue #5417 (min resolution in AVC mode) */
-#define AVC_MIN_DESKTOP_WIDTH 642
+/**
+ * Constants to workaround FreeRDP issue #5417 (min resolution in AVC mode)
+ * Must be 4 aligned (multiple of 4).
+ * We should instead check destRect and be sure the condition is met
+ * See https://gitlab.com/Remmina/Remmina/-/issues/2507 for a deep discussion
+ */
+#define AVC_MIN_DESKTOP_WIDTH 644
 #define AVC_MIN_DESKTOP_HEIGHT 480
 
 typedef struct rf_context rfContext;
@@ -173,11 +178,16 @@ struct remmina_plugin_rdp_event {
 			CLIPRDR_FORMAT_DATA_REQUEST *pFormatDataRequest;
 		} clipboard_formatdatarequest;
 		struct {
+			gint    Flags;
+			gint    Left;
+			gint    Top;
 			gint	width;
 			gint	height;
 			gint	desktopOrientation;
 			gint	desktopScaleFactor;
 			gint	deviceScaleFactor;
+			gint    physicalWidth;
+			gint    physicalHeight;
 		} monitor_layout;
 	};
 };
@@ -298,6 +308,7 @@ struct rf_context {
 
 	gboolean		connected;
 	gboolean		is_reconnecting;
+	gboolean		stop_reconnecting_requested;
 	/* orphaned: rf_context has still one or more libfreerdp thread active,
 	 * but no longer maintained by an open RemminaProtocolWidget/tab.
 	 * When the orphaned thread terminates, we must cleanup rf_context.
