@@ -52,6 +52,10 @@ static void remmina_plugin_spice_main_channel_event_cb(SpiceChannel *, SpiceChan
 static void remmina_plugin_spice_agent_connected_event_cb(SpiceChannel *, RemminaProtocolWidget *);
 static void remmina_plugin_spice_display_ready_cb(GObject *, GParamSpec *, RemminaProtocolWidget *);
 static void remmina_plugin_spice_update_scale_mode(RemminaProtocolWidget *);
+static gboolean remmina_plugin_spice_session_open_fd(RemminaProtocolWidget *gp, gint fd) __attribute__ ((unused));
+static gboolean remmina_plugin_spice_channel_open_fd(SpiceChannel *, int with_tls, RemminaProtocolWidget *) __attribute__ ((unused));
+//static gboolean remmina_plugin_spice_session_open_fd(RemminaProtocolWidget *gp, gint fd);
+//static gboolean remmina_plugin_spice_channel_open_fd(SpiceChannel *, int with_tls, RemminaProtocolWidget *);
 
 void remmina_plugin_spice_select_usb_devices(RemminaProtocolWidget *);
 #ifdef SPICE_GTK_CHECK_VERSION
@@ -60,6 +64,54 @@ void remmina_plugin_spice_file_transfer_new_cb(SpiceMainChannel *, SpiceFileTran
 #  endif        /* SPICE_GTK_CHECK_VERSION(0, 31, 0) */
 #endif          /* SPICE_GTK_CHECK_VERSION */
 
+gchar* str_replace(const gchar *string, const gchar *search, const gchar *replacement)
+{
+	TRACE_CALL(__func__);
+	gchar *str, **arr;
+
+	g_return_val_if_fail(string != NULL, NULL);
+	g_return_val_if_fail(search != NULL, NULL);
+
+	if (replacement == NULL)
+		replacement = "";
+
+	arr = g_strsplit(string, search, -1);
+	if (arr != NULL && arr[0] != NULL)
+		str = g_strjoinv(replacement, arr);
+	else
+		str = g_strdup(string);
+
+	g_strfreev(arr);
+	return str;
+}
+
+static gboolean
+remmina_plugin_spice_session_open_fd(RemminaProtocolWidget *gp, int fd)
+{
+	TRACE_CALL(__func__);
+	RemminaPluginSpiceData *gpdata = GET_PLUGIN_DATA(gp);
+
+	g_return_val_if_fail(gpdata != NULL, FALSE);
+
+	REMMINA_PLUGIN_DEBUG("Opening spice session for FD: %d", fd);
+	return spice_session_open_fd(gpdata->session, fd);
+}
+
+static gboolean
+remmina_plugin_spice_channel_open_fd(SpiceChannel *channel, int with_tls G_GNUC_UNUSED, RemminaProtocolWidget *gp)
+{
+	TRACE_CALL(__func__);
+	RemminaPluginSpiceData *gpdata = GET_PLUGIN_DATA(gp);
+	g_return_val_if_fail(gpdata != NULL, FALSE);
+
+	gint id, type;
+
+	g_object_get(channel, "channel-id", &id, "channel-type", &type, NULL);
+	REMMINA_PLUGIN_DEBUG ("Opening channel %p %s %d with FD: %d", channel, g_type_name(G_OBJECT_TYPE(channel)), id, gpdata->fd);
+
+	return spice_channel_open_fd(channel, gpdata->fd);
+}
+
 static void remmina_plugin_spice_init(RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
@@ -109,31 +161,50 @@ static gboolean remmina_plugin_spice_open_connection(RemminaProtocolWidget *gp)
 		return FALSE;
 	}
 
-	remmina_plugin_service->get_server_port(tunnel,
-		XSPICE_DEFAULT_PORT,
-		&host,
-		&port);
+	/**-START- UNIX socket
+	* if(strstr(g_strdup(tunnel), "unix:///") != NULL) {
+	* 	REMMINA_PLUGIN_DEBUG("Tunnel contain unix:// -> %s", tunnel);
+	* 	gchar *val = str_replace (tunnel, "unix://", "");
+	* 	REMMINA_PLUGIN_DEBUG("tunnel after cleaning = %s", val);
+	* 	//gchar *val = g_strdup(remmina_plugin_service->file_get_string(remminafile, "server"));
+	* 	g_object_set(gpdata->session, "unix-path", val, NULL);
+	* 	gint fd = remmina_plugin_service->open_unix_sock(val);
+	* 	REMMINA_PLUGIN_DEBUG("Unix socket fd: %d", fd);
+	* 	gpdata->fd = fd;
+	* 	if (fd > 0)
+	* 		remmina_plugin_spice_session_open_fd (gp, fd);
+	* 	g_free(val);
+	*
+	* } else {
+	*/
 
-	g_object_set(gpdata->session, "host", host, NULL);
-	g_free(host);
-	g_free(tunnel);
 
-	/* Unencrypted connection */
-	if (!remmina_plugin_service->file_get_int(remminafile, "usetls", FALSE)) {
-		g_object_set(gpdata->session, "port", g_strdup_printf("%i", port), NULL);
-	}
-	/* TLS encrypted connection */
-	else{
-		g_object_set(gpdata->session, "tls_port", g_strdup_printf("%i", port), NULL);
+		remmina_plugin_service->get_server_port(tunnel,
+				XSPICE_DEFAULT_PORT,
+				&host,
+				&port);
 
-		/* Server CA certificate */
-		cacert = remmina_plugin_service->file_get_string(remminafile, "cacert");
-		if (cacert) {
-			g_object_set(gpdata->session, "ca-file", cacert, NULL);
+		g_object_set(gpdata->session, "host", host, NULL);
+		g_free(host);
+		g_free(tunnel);
+
+		/* Unencrypted connection */
+		if (!remmina_plugin_service->file_get_int(remminafile, "usetls", FALSE)) {
+			g_object_set(gpdata->session, "port", g_strdup_printf("%i", port), NULL);
 		}
-	}
+		/* TLS encrypted connection */
+		else{
+			g_object_set(gpdata->session, "tls_port", g_strdup_printf("%i", port), NULL);
 
-	spice_session_connect(gpdata->session);
+			/* Server CA certificate */
+			cacert = remmina_plugin_service->file_get_string(remminafile, "cacert");
+			if (cacert) {
+				g_object_set(gpdata->session, "ca-file", cacert, NULL);
+			}
+		}
+
+		spice_session_connect(gpdata->session);
+	/** } -END- UNIX socket */
 
 	/*
 	 * FIXME: Add a waiting loop until the g_signal "channel-event" occurs.
@@ -185,11 +256,21 @@ static void remmina_plugin_spice_channel_new_cb(SpiceSession *session, SpiceChan
 {
 	TRACE_CALL(__func__);
 
-	gint id;
+	gint id, type;
 	RemminaPluginSpiceData *gpdata = GET_PLUGIN_DATA(gp);
 	RemminaFile *remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
 
-	g_object_get(channel, "channel-id", &id, NULL);
+	g_return_if_fail(gpdata != NULL);
+
+	/**
+	* g_signal_connect(channel,
+	*	"open-fd",
+	*	G_CALLBACK(remmina_plugin_spice_channel_open_fd),
+	*	gp);
+	*/
+
+	g_object_get(channel, "channel-id", &id, "channel-type", &type, NULL);
+	REMMINA_PLUGIN_DEBUG ("New spice channel %p %s %d", channel, g_type_name(G_OBJECT_TYPE(channel)), id);
 
 	if (SPICE_IS_MAIN_CHANNEL(channel)) {
 		gpdata->main_channel = SPICE_MAIN_CHANNEL(channel);
@@ -229,13 +310,22 @@ static void remmina_plugin_spice_channel_new_cb(SpiceSession *session, SpiceChan
 
 	}
 
+	if (SPICE_IS_INPUTS_CHANNEL(channel)) {
+		REMMINA_PLUGIN_DEBUG("New inputs channel");
+	}
 	if (SPICE_IS_PLAYBACK_CHANNEL(channel)) {
+		REMMINA_PLUGIN_DEBUG("New audio channel");
 		if (remmina_plugin_service->file_get_int(remminafile, "enableaudio", FALSE)) {
 			gpdata->audio = spice_audio_get(gpdata->session, NULL);
 		}
 	}
 
+	if (SPICE_IS_USBREDIR_CHANNEL(channel)) {
+		REMMINA_PLUGIN_DEBUG("New usbredir channel");
+	}
+
 	if (SPICE_IS_WEBDAV_CHANNEL(channel)) {
+		REMMINA_PLUGIN_DEBUG("New webdav channel");
 		if (remmina_plugin_service->file_get_string(remminafile, "sharefolder")) {
 			spice_channel_connect(channel);
 		}
@@ -348,8 +438,6 @@ static void remmina_plugin_spice_display_ready_cb(GObject *display, GParamSpec *
 	g_object_get(display, "ready", &ready, NULL);
 
 	if (ready) {
-		SpiceVideoCodecType videocodec;
-		SpiceImageCompression imagecompression;
 		RemminaPluginSpiceData *gpdata = GET_PLUGIN_DATA(gp);
 		RemminaFile *remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
 
@@ -363,8 +451,11 @@ static void remmina_plugin_spice_display_ready_cb(GObject *display, GParamSpec *
 			"resize-guest", (scaleMode == REMMINA_PROTOCOL_WIDGET_SCALE_MODE_DYNRES),
 			NULL);
 
-		videocodec = remmina_plugin_service->file_get_int(remminafile, "videocodec", 0);
+#ifdef SPICE_GTK_CHECK_VERSION
+#  if SPICE_GTK_CHECK_VERSION(0, 34, 0)
+		SpiceVideoCodecType videocodec = remmina_plugin_service->file_get_int(remminafile, "videocodec", 0);
 		if (videocodec) {
+#    if SPICE_GTK_CHECK_VERSION(0, 38, 0)
 			GError *err = NULL;
 			guint i;
 
@@ -388,13 +479,32 @@ static void remmina_plugin_spice_display_ready_cb(GObject *display, GParamSpec *
 			}
 
 			g_clear_pointer(&preferred_codecs, g_array_unref);
-		}
 
-		imagecompression = remmina_plugin_service->file_get_int(remminafile, "imagecompression", 0);
+#    elif SPICE_GTK_CHECK_VERSION(0, 35, 0)
+			spice_display_channel_change_preferred_video_codec_type(SPICE_CHANNEL(gpdata->display_channel),
+					videocodec);
+#    else
+			spice_display_change_preferred_video_codec_type(SPICE_CHANNEL(gpdata->display_channel),
+					videocodec);
+#    endif
+		}
+#  endif
+#endif
+
+#ifdef SPICE_GTK_CHECK_VERSION
+#  if SPICE_GTK_CHECK_VERSION(0, 31, 0)
+		SpiceImageCompression imagecompression = remmina_plugin_service->file_get_int(remminafile, "imagecompression", 0);
 		if (imagecompression) {
+#    if SPICE_GTK_CHECK_VERSION(0, 35, 0)
 			spice_display_channel_change_preferred_compression(SPICE_CHANNEL(gpdata->display_channel),
 				imagecompression);
+#    else
+			spice_display_change_preferred_compression(SPICE_CHANNEL(gpdata->display_channel),
+				imagecompression);
+#    endif
 		}
+#  endif
+#endif
 
 		gtk_container_add(GTK_CONTAINER(gp), GTK_WIDGET(display));
 		gtk_widget_show(GTK_WIDGET(display));
@@ -496,6 +606,8 @@ static void remmina_plugin_spice_call_feature(RemminaProtocolWidget *gp, const R
 	}
 }
 
+#ifdef SPICE_GTK_CHECK_VERSION
+#  if SPICE_GTK_CHECK_VERSION(0, 34, 0)
 /* Array of key/value pairs for prefered video codec
  * Key - SpiceVideoCodecType (spice/enums.h)
  */
@@ -509,7 +621,11 @@ static gpointer videocodec_list[] =
 	"5",	"h265",
 	NULL
 };
+#  endif
+#endif
 
+#ifdef SPICE_GTK_CHECK_VERSION
+#  if SPICE_GTK_CHECK_VERSION(0, 31, 0)
 /* Array of key/value pairs for prefered video codec
  * Key - SpiceImageCompression (spice/enums.h)
  */
@@ -525,9 +641,15 @@ static gpointer imagecompression_list[] =
 	"7",	"LZ4",
 	NULL
 };
+#  endif
+#endif
 
+#ifdef SPICE_GTK_CHECK_VERSION
+#  if SPICE_GTK_CHECK_VERSION(0, 34, 0)
 static gchar disablegstvideooverlay_tooltip[] =
 	N_("Disable video overlay if videos are not displayed properly.\n");
+#  endif
+#endif
 
 /* Array of RemminaProtocolSetting for basic settings.
  * Each item is composed by:
@@ -559,9 +681,15 @@ static const RemminaProtocolSetting remmina_plugin_spice_basic_settings[] =
  */
 static const RemminaProtocolSetting remmina_plugin_spice_advanced_settings[] =
 {
+#ifdef SPICE_GTK_CHECK_VERSION
+#  if SPICE_GTK_CHECK_VERSION(0, 35, 0)
 	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	"videocodec",	    N_("Prefered video codec"),		FALSE, videocodec_list, NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"disablegstvideooverlay",	    N_("Turn off GStreamer overlay"),		FALSE,	NULL,	disablegstvideooverlay_tooltip},
+#  endif
+#  if SPICE_GTK_CHECK_VERSION(0, 31, 0)
 	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	"imagecompression",	    N_("Prefered image compression"),		FALSE, imagecompression_list, NULL},
+#  endif
+#endif
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"disableclipboard",	    N_("No clipboard sync"),		TRUE,	NULL,	NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"disablepasswordstoring",   N_("Forget passwords after use"),		TRUE,	NULL,	NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"enableaudio",		    N_("Enable audio channel"),			TRUE,	NULL,	NULL},
@@ -603,7 +731,9 @@ static RemminaProtocolPlugin remmina_plugin_spice =
 	remmina_plugin_spice_query_feature,                                     // Query for available features
 	remmina_plugin_spice_call_feature,                                      // Call a feature
 	remmina_plugin_spice_keystroke,                                         // Send a keystroke
-	NULL                                                                    // No screenshot support available
+	NULL,                                                                   // No screenshot support available
+	NULL,                                                                   // RCW map event
+	NULL                                                                    // RCW unmap event
 };
 
 void remmina_plugin_spice_remove_list_option(gpointer *option_list, const gchar *option_to_remove) {
@@ -625,6 +755,8 @@ void remmina_plugin_spice_remove_list_option(gpointer *option_list, const gchar
 	*dst = NULL;
 }
 
+#ifdef SPICE_GTK_CHECK_VERSION
+#  if SPICE_GTK_CHECK_VERSION(0, 31, 0)
 gboolean remmina_plugin_spice_is_lz4_supported() {
 	gboolean result = FALSE;
 	GOptionContext *context;
@@ -659,6 +791,8 @@ gboolean remmina_plugin_spice_is_lz4_supported() {
 
 	return result;
 }
+#  endif
+#endif
 
 G_MODULE_EXPORT gboolean
 remmina_plugin_entry(RemminaPluginService *service)
@@ -669,11 +803,15 @@ remmina_plugin_entry(RemminaPluginService *service)
 	bindtextdomain(GETTEXT_PACKAGE, REMMINA_RUNTIME_LOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 
+#ifdef SPICE_GTK_CHECK_VERSION
+#  if SPICE_GTK_CHECK_VERSION(0, 31, 0)
 	if (!remmina_plugin_spice_is_lz4_supported()) {
 		char key_str[10];
 		sprintf(key_str, "%d", SPICE_IMAGE_COMPRESSION_LZ4);
 		remmina_plugin_spice_remove_list_option(imagecompression_list, key_str);
 	}
+#  endif
+#endif
 
 	if (!service->register_plugin((RemminaPlugin*)&remmina_plugin_spice)) {
 		return FALSE;
