@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2010 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,13 +48,13 @@ G_BEGIN_DECLS
 typedef struct _RemminaFTPClientPriv RemminaFTPClientPriv;
 
 typedef struct _RemminaFTPClient {
-	GtkVBox			vbox;
+	GtkBox			vbox;
 
 	RemminaFTPClientPriv *	priv;
 } RemminaFTPClient;
 
 typedef struct _RemminaFTPClientClass {
-	GtkVBoxClass parent_class;
+	GtkBoxClass parent_class;
 
 	void (*open_dir)(RemminaFTPClient *client);
 	void (*new_task)(RemminaFTPClient *client);
