@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -84,6 +84,7 @@ struct _RemminaProtocolWidgetPriv {
 
 	gint			profile_remote_width;
 	gint			profile_remote_height;
+	gint			multimon;
 
 	RemminaMessagePanel *	connect_message_panel;
 	RemminaMessagePanel *	listen_message_panel;
@@ -114,6 +115,7 @@ enum {
 	DISCONNECT_SIGNAL,
 	DESKTOP_RESIZE_SIGNAL,
 	UPDATE_ALIGN_SIGNAL,
+	LOCK_DYNRES_SIGNAL,
 	UNLOCK_DYNRES_SIGNAL,
 	LAST_SIGNAL
 };
@@ -141,6 +143,9 @@ static void remmina_protocol_widget_class_init(RemminaProtocolWidgetClass *klass
 	remmina_protocol_widget_signals[UPDATE_ALIGN_SIGNAL] = g_signal_new("update-align", G_TYPE_FROM_CLASS(klass),
 									    G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET(RemminaProtocolWidgetClass, update_align), NULL, NULL,
 									    g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+	remmina_protocol_widget_signals[LOCK_DYNRES_SIGNAL] = g_signal_new("lock-dynres", G_TYPE_FROM_CLASS(klass),
+									     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET(RemminaProtocolWidgetClass, lock_dynres), NULL, NULL,
+									     g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 	remmina_protocol_widget_signals[UNLOCK_DYNRES_SIGNAL] = g_signal_new("unlock-dynres", G_TYPE_FROM_CLASS(klass),
 									     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET(RemminaProtocolWidgetClass, unlock_dynres), NULL, NULL,
 									     g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
@@ -157,7 +162,7 @@ static void remmina_protocol_widget_close_all_tunnels(RemminaProtocolWidget *gp)
 #ifdef HAVE_LIBSSH
 			remmina_ssh_tunnel_free((RemminaSSHTunnel *)gp->priv->ssh_tunnels->pdata[i]);
 #else
-			g_debug ("LibSSH support turned off, no need to free SSH tunnel data");
+			REMMINA_DEBUG ("LibSSH support turned off, no need to free SSH tunnel data");
 #endif
 		}
 	}
@@ -240,6 +245,8 @@ void remmina_protocol_widget_open_connection_real(gpointer data)
 	TRACE_CALL(__func__);
 	RemminaProtocolWidget *gp = REMMINA_PROTOCOL_WIDGET(data);
 
+	REMMINA_DEBUG("Opening connection");
+
 	RemminaProtocolPlugin *plugin;
 	RemminaProtocolFeature *feature;
 	gint num_plugin;
@@ -269,17 +276,20 @@ void remmina_protocol_widget_open_connection_real(gpointer data)
 			feature += num_plugin;
 		}
 #ifdef HAVE_LIBSSH
+	REMMINA_DEBUG("Have SSH");
 		if (num_ssh) {
 			feature->type = REMMINA_PROTOCOL_FEATURE_TYPE_TOOL;
 			feature->id = REMMINA_PROTOCOL_FEATURE_TOOL_SSH;
 			feature->opt1 = _("Connect via SSH from a new terminal");
 			feature->opt2 = "utilities-terminal";
+			feature->opt3 = NULL;
 			feature++;
 
 			feature->type = REMMINA_PROTOCOL_FEATURE_TYPE_TOOL;
 			feature->id = REMMINA_PROTOCOL_FEATURE_TOOL_SFTP;
 			feature->opt1 = _("Open SFTP transfer…");
 			feature->opt2 = "folder-remote";
+			feature->opt3 = NULL;
 			feature++;
 		}
 		feature->type = REMMINA_PROTOCOL_FEATURE_TYPE_END;
@@ -313,7 +323,8 @@ void remmina_protocol_widget_open_connection(RemminaProtocolWidget *gp)
 	rco_destroy_message_panel(gp->cnnobj, mp);
 
 	name = remmina_file_get_string(gp->priv->remmina_file, "name");
-	s = g_strdup_printf(_("Connecting to \"%s\"…"), (name ? name : "*"));
+	// TRANSLATORS: “%s” is a placeholder for the connection profile name
+	s = g_strdup_printf(_("Connecting to “%s”…"), (name ? name : "*"));
 
 	mp = remmina_message_panel_new();
 	remmina_message_panel_setup_progress(mp, s, cancel_open_connection_cb, gp);
@@ -398,6 +409,14 @@ void remmina_protocol_widget_update_align(RemminaProtocolWidget *gp)
 	g_idle_add(update_align, (gpointer)gp);
 }
 
+static gboolean lock_dynres(gpointer data)
+{
+	TRACE_CALL(__func__);
+	RemminaProtocolWidget *gp = (RemminaProtocolWidget *)data;
+	g_signal_emit_by_name(G_OBJECT(gp), "lock-dynres");
+	return G_SOURCE_REMOVE;
+}
+
 static gboolean unlock_dynres(gpointer data)
 {
 	TRACE_CALL(__func__);
@@ -406,6 +425,13 @@ static gboolean unlock_dynres(gpointer data)
 	return G_SOURCE_REMOVE;
 }
 
+void remmina_protocol_widget_lock_dynres(RemminaProtocolWidget *gp)
+{
+	/* Called by the plugin to do updates on rcw */
+	TRACE_CALL(__func__);
+	g_idle_add(lock_dynres, (gpointer)gp);
+}
+
 void remmina_protocol_widget_unlock_dynres(RemminaProtocolWidget *gp)
 {
 	/* Called by the plugin to do updates on rcw */
@@ -554,8 +580,119 @@ void remmina_protocol_widget_send_keystrokes(RemminaProtocolWidget *gp, GtkMenuI
 	return;
 }
 
+/**
+ * Send to the plugin some keystrokes from the content of the clipboard
+ * This is a copy of remmina_protocol_widget_send_keystrokes but it uses the clipboard content
+ * get from remmina_protocol_widget_send_clipboard
+ * Probably we don't need the replacement table.
+ */
+void remmina_protocol_widget_send_clip_strokes(GtkClipboard *clipboard, const gchar *clip_text, gpointer data)
+{
+	TRACE_CALL(__func__);
+	RemminaProtocolWidget *gp = REMMINA_PROTOCOL_WIDGET(data);
+	gchar *text = g_strdup(clip_text);
+	guint *keyvals;
+	gint i;
+	GdkKeymap *keymap = gdk_keymap_get_for_display(gdk_display_get_default());
+	gunichar character;
+	guint keyval;
+	GdkKeymapKey *keys;
+	gint n_keys;
+	/* Single keystroke replace */
+	typedef struct _KeystrokeReplace {
+		gchar * search;
+		gchar * replace;
+		guint	keyval;
+	} KeystrokeReplace;
+	/* Special characters to replace */
+	KeystrokeReplace text_replaces[] =
+	{
+		{ "\\n",  "\n", GDK_KEY_Return	  },
+		{ "\\t",  "\t", GDK_KEY_Tab	  },
+		{ "\\b",  "\b", GDK_KEY_BackSpace },
+		{ "\\e",  "\e", GDK_KEY_Escape	  },
+		{ "\\\\", "\\", GDK_KEY_backslash },
+		{ NULL,	  NULL, 0		  }
+	};
+	if (remmina_protocol_widget_plugin_receives_keystrokes(gp)) {
+		if(text) {
+			/* Replace special characters */
+			for (i = 0; text_replaces[i].replace; i++) {
+				REMMINA_DEBUG("Text clipboard before replacement is \'%s\'", text);
+				text = g_strdup(remmina_public_str_replace_in_place(text,
+							text_replaces[i].search,
+							text_replaces[i].replace));
+				REMMINA_DEBUG("Text clipboard after replacement is \'%s\'", text);
+			}
+			gchar *iter = g_strdup(text);
+			keyvals = (guint *)g_malloc(strlen(text));
+			while (TRUE) {
+				/* Process each character in the keystrokes */
+				character = g_utf8_get_char_validated(iter, -1);
+				if (character == 0)
+					break;
+				keyval = gdk_unicode_to_keyval(character);
+				/* Replace all the special character with its keyval */
+				for (i = 0; text_replaces[i].replace; i++) {
+					if (character == text_replaces[i].replace[0]) {
+						keys = g_new0(GdkKeymapKey, 1);
+						keyval = text_replaces[i].keyval;
+						/* A special character was generated, no keyval lookup needed */
+						character = 0;
+						break;
+					}
+				}
+				/* Decode character if it’s not a special character */
+				if (character) {
+					/* get keyval without modifications */
+					if (!gdk_keymap_get_entries_for_keyval(keymap, keyval, &keys, &n_keys)) {
+						g_warning("keyval 0x%04x has no keycode!", keyval);
+						iter = g_utf8_find_next_char(iter, NULL);
+						continue;
+					}
+				}
+				/* Add modifier keys */
+				n_keys = 0;
+				if (keys->level & 1)
+					keyvals[n_keys++] = GDK_KEY_Shift_L;
+				if (keys->level & 2)
+					keyvals[n_keys++] = GDK_KEY_Alt_R;
+				/*
+				 * @fixme heap buffer overflow
+				 * In some cases, for example sending \t as the only sequence
+				 * may lead to a buffer overflow
+				 */
+				keyvals[n_keys++] = keyval;
+				/* Send keystroke to the plugin */
+				gp->priv->plugin->send_keystrokes(gp, keyvals, n_keys);
+				g_free(keys);
+				/* Process next character in the keystrokes */
+				iter = g_utf8_find_next_char(iter, NULL);
+			}
+			g_free(keyvals);
+		}
+		g_free(text);
+	}
+	return;
+}
+
+void remmina_protocol_widget_send_clipboard(RemminaProtocolWidget *gp, GtkMenuItem *widget)
+{
+	TRACE_CALL(__func__);
+	GtkClipboard *clipboard;
+
+	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+
+	/* Request the contents of the clipboard, contents_received will be
+	   called when we do get the contents.
+	   */
+	gtk_clipboard_request_text (clipboard,
+			remmina_protocol_widget_send_clip_strokes, gp);
+}
+
 gboolean remmina_protocol_widget_plugin_screenshot(RemminaProtocolWidget *gp, RemminaPluginScreenshotData *rpsd)
 {
+	TRACE_CALL(__func__);
 	if (!gp->priv->plugin->get_plugin_screenshot) {
 		REMMINA_DEBUG("plugin screenshot function is not implemented, using core Remmina functionality");
 		return FALSE;
@@ -564,6 +701,30 @@ gboolean remmina_protocol_widget_plugin_screenshot(RemminaProtocolWidget *gp, Re
 	return gp->priv->plugin->get_plugin_screenshot(gp, rpsd);
 }
 
+gboolean remmina_protocol_widget_map_event(RemminaProtocolWidget *gp)
+{
+	TRACE_CALL(__func__);
+	if (!gp->priv->plugin->map_event) {
+		REMMINA_DEBUG("Map plugin function not implemented");
+		return FALSE;
+	}
+
+	REMMINA_DEBUG ("Calling plugin mapping function");
+	return gp->priv->plugin->map_event(gp);
+}
+
+gboolean remmina_protocol_widget_unmap_event(RemminaProtocolWidget *gp)
+{
+	TRACE_CALL(__func__);
+	if (!gp->priv->plugin->get_plugin_screenshot) {
+		REMMINA_DEBUG("Unmap plugin function not implemented");
+		return FALSE;
+	}
+
+	REMMINA_DEBUG ("Calling plugin unmapping function");
+	return gp->priv->plugin->unmap_event(gp);
+}
+
 void remmina_protocol_widget_emit_signal(RemminaProtocolWidget *gp, const gchar *signal_name)
 {
 	TRACE_CALL(__func__);
@@ -740,17 +901,74 @@ static RemminaSSHTunnel* remmina_protocol_widget_init_tunnel(RemminaProtocolWidg
 	gint ret;
 	gchar *msg;
 	RemminaMessagePanel *mp;
+	gboolean partial = FALSE;
+	gboolean cont = FALSE;
 
 	tunnel = remmina_ssh_tunnel_new_from_file(gp->priv->remmina_file);
 
-	g_debug ("[RPW] %s creating SSH tunnel to \"%s\" via SSH…", __func__, REMMINA_SSH(tunnel)->server);
-	msg = g_strdup_printf(_("Connecting to \"%s\" via SSH…"), REMMINA_SSH(tunnel)->server);
+	REMMINA_DEBUG ("Creating SSH tunnel to “%s” via SSH…", REMMINA_SSH(tunnel)->server);
+	// TRANSLATORS: “%s” is a placeholder for an hostname or an IP address.
+	msg = g_strdup_printf(_("Connecting to “%s” via SSH…"), REMMINA_SSH(tunnel)->server);
 
 	mp = remmina_protocol_widget_mpprogress(gp->cnnobj, msg, cancel_init_tunnel_cb, NULL);
 	g_free(msg);
 
+
+
+	while (1) {
+		if (!partial) {
+			if (!remmina_ssh_init_session(REMMINA_SSH(tunnel))) {
+				REMMINA_DEBUG("SSH Tunnel init session error: %s", REMMINA_SSH(tunnel)->error);
+				remmina_protocol_widget_set_error(gp, REMMINA_SSH(tunnel)->error);
+				// exit the loop here: OK
+				break;
+			}
+		}
+
+		ret = remmina_ssh_auth_gui(REMMINA_SSH(tunnel), gp, gp->priv->remmina_file);
+		REMMINA_DEBUG ("Tunnel auth returned %d", ret);
+		switch (ret) {
+			case REMMINA_SSH_AUTH_SUCCESS:
+				REMMINA_DEBUG("Authentication success");
+				break;
+			case REMMINA_SSH_AUTH_PARTIAL:
+				REMMINA_DEBUG("Continue with the next auth method");
+				partial = TRUE;
+				// Continue the loop: OK
+				continue;
+				break;
+			case REMMINA_SSH_AUTH_RECONNECT:
+				REMMINA_DEBUG("Reconnecting…");
+				if (REMMINA_SSH(tunnel)->session) {
+					ssh_disconnect(REMMINA_SSH(tunnel)->session);
+					ssh_free(REMMINA_SSH(tunnel)->session);
+					REMMINA_SSH(tunnel)->session = NULL;
+				}
+				g_free(REMMINA_SSH(tunnel)->callback);
+				// Continue the loop: OK
+				continue;
+				break;
+			case REMMINA_SSH_AUTH_USERCANCEL:
+				REMMINA_DEBUG("Interrupted by the user");
+				// exit the loop here: OK
+				goto BREAK;
+				break;
+			default:
+				REMMINA_DEBUG("Error during the authentication: %s", REMMINA_SSH(tunnel)->error);
+				remmina_protocol_widget_set_error(gp, REMMINA_SSH(tunnel)->error);
+				// exit the loop here: OK
+				goto BREAK;
+		}
+
+
+		cont = TRUE;
+		break;
+	}
+
+#if 0
+
 	if (!remmina_ssh_init_session(REMMINA_SSH(tunnel))) {
-		REMMINA_DEBUG ("%s Cannot init SSH session with tunnel struct", __func__);
+		REMMINA_DEBUG ("Cannot init SSH session with tunnel struct");
 		remmina_protocol_widget_set_error(gp, REMMINA_SSH(tunnel)->error);
 		remmina_ssh_tunnel_free(tunnel);
 		return NULL;
@@ -765,6 +983,13 @@ static RemminaSSHTunnel* remmina_protocol_widget_init_tunnel(RemminaProtocolWidg
 		return NULL;
 	}
 
+#endif
+
+BREAK:
+	if (!cont) {
+		remmina_ssh_tunnel_free(tunnel);
+		return NULL;
+	}
 	remmina_protocol_widget_mpdestroy(gp->cnnobj, mp);
 
 	return tunnel;
@@ -803,7 +1028,7 @@ static gboolean remmina_protocol_widget_tunnel_destroy(RemminaSSHTunnel *tunnel,
 
 	if (found) {
 #ifdef HAVE_LIBSSH
-		g_debug("[RPW] tunnel with idx %u has been disconnected", idx);
+		REMMINA_DEBUG("[Tunnel with idx %u has been disconnected", idx);
 		remmina_ssh_tunnel_free(tunnel);
 #endif
 		g_ptr_array_remove(gp->priv->ssh_tunnels, tunnel);
@@ -820,12 +1045,10 @@ gchar *remmina_protocol_widget_start_direct_tunnel(RemminaProtocolWidget *gp, gi
 	TRACE_CALL(__func__);
 	const gchar *server;
 	const gchar *ssh_tunnel_server;
-	//const gchar *proto;
 	gchar *ssh_tunnel_host, *srv_host, *dest;
 	gint srv_port, ssh_tunnel_port;
 
-	g_debug ("SSH tunnel initialization…");
-	//proto = remmina_file_get_string(gp->priv->remmina_file, "protocol");
+	REMMINA_DEBUG ("SSH tunnel initialization…");
 
 	server = remmina_file_get_string(gp->priv->remmina_file, "server");
 	ssh_tunnel_server = remmina_file_get_string(gp->priv->remmina_file, "ssh_tunnel_server");
@@ -833,9 +1056,16 @@ gchar *remmina_protocol_widget_start_direct_tunnel(RemminaProtocolWidget *gp, gi
 	if (!server)
 		return g_strdup("");
 
+	if(strstr(g_strdup(server), "unix:///") != NULL) {
+		REMMINA_DEBUG ("%s is a UNIX socket", server);
+		return g_strdup(server);
+	}
+
+	REMMINA_DEBUG ("Calling remmina_public_get_server_port");
 	remmina_public_get_server_port(server, default_port, &srv_host, &srv_port);
+	REMMINA_DEBUG ("Calling remmina_public_get_server_port (tunnel)");
 	remmina_public_get_server_port(ssh_tunnel_server, 22, &ssh_tunnel_host, &ssh_tunnel_port);
-	g_debug ("server: %s, port: %d", srv_host, srv_port);
+	REMMINA_DEBUG ("server: %s, port: %d", srv_host, srv_port);
 
 	if (port_plus && srv_port < 100)
 		/* Protocols like VNC supports using instance number :0, :1, etc. as port number. */
@@ -857,12 +1087,13 @@ gchar *remmina_protocol_widget_start_direct_tunnel(RemminaProtocolWidget *gp, gi
 	if (!tunnel) {
 		g_free(srv_host);
 		g_free(ssh_tunnel_host);
-		REMMINA_DEBUG ("%s remmina_protocol_widget_init_tunnel failed with error is %s",
-				 __func__, remmina_protocol_widget_get_error_message(gp));
+		REMMINA_DEBUG ("remmina_protocol_widget_init_tunnel failed with error is %s",
+				remmina_protocol_widget_get_error_message(gp));
 		return NULL;
 	}
 
-	msg = g_strdup_printf(_("Connecting to \"%s\" via SSH…"), server);
+	// TRANSLATORS: “%s” is a placeholder for an hostname or an IP address.
+	msg = g_strdup_printf(_("Connecting to “%s” via SSH…"), server);
 	mp = remmina_protocol_widget_mpprogress(gp->cnnobj, msg, cancel_start_direct_tunnel_cb, NULL);
 	g_free(msg);
 
@@ -873,7 +1104,7 @@ gchar *remmina_protocol_widget_start_direct_tunnel(RemminaProtocolWidget *gp, gi
 		srv_host = g_strdup("127.0.0.1");
 	}
 
-	g_debug ("%s: starting tunnel to: %s, port: %d", __func__, ssh_tunnel_host, ssh_tunnel_port);
+	REMMINA_DEBUG ("Starting tunnel to: %s, port: %d", ssh_tunnel_host, ssh_tunnel_port);
 	if (!remmina_ssh_tunnel_open(tunnel, srv_host, srv_port, remmina_pref.sshtunnel_port)) {
 		g_free(srv_host);
 		g_free(ssh_tunnel_host);
@@ -925,6 +1156,7 @@ gboolean remmina_protocol_widget_start_reverse_tunnel(RemminaProtocolWidget *gp,
 	if (!(tunnel = remmina_protocol_widget_init_tunnel(gp)))
 		return FALSE;
 
+	// TRANSLATORS: “%i” is a placeholder for a TCP port number.
 	msg = g_strdup_printf(_("Awaiting incoming SSH connection on port %i…"), remmina_file_get_int(gp->priv->remmina_file, "listenport", 0));
 	mp = remmina_protocol_widget_mpprogress(gp->cnnobj, msg, cancel_start_reverse_tunnel_cb, NULL);
 	g_free(msg);
@@ -976,12 +1208,14 @@ gboolean remmina_protocol_widget_ssh_exec(RemminaProtocolWidget *gp, gboolean wa
 				ret = TRUE;
 				break;
 			case 127:
+				// TRANSLATORS: “%s” is a place holder for a unix command path.
 				remmina_ssh_set_application_error(REMMINA_SSH(tunnel),
-								  _("The \"%s\" command is not available on the SSH server."), cmd);
+								  _("The “%s” command is not available on the SSH server."), cmd);
 				break;
 			default:
+				// TRANSLATORS: “%s” is a place holder for a unix command path. “%i” is a placeholder for an error code number.
 				remmina_ssh_set_application_error(REMMINA_SSH(tunnel),
-								  _("Could not run the \"%s\" command on the SSH server (status = %i)."), cmd, status);
+								  _("Could not run the “%s” command on the SSH server (status = %i)."), cmd, status);
 				break;
 			}
 		} else {
@@ -1013,6 +1247,7 @@ static gboolean remmina_protocol_widget_xport_tunnel_init_callback(RemminaSSHTun
 	gint port;
 	gboolean ret;
 
+	REMMINA_DEBUG ("Calling remmina_public_get_server_port");
 	remmina_public_get_server_port(remmina_file_get_string(gp->priv->remmina_file, "server"), 177, &server, &port);
 	ret = ((RemminaXPortTunnelInitFunc)gp->priv->init_func)(gp,
 								tunnel->remotedisplay, (tunnel->bindlocalhost ? "localhost" : server), port);
@@ -1057,6 +1292,7 @@ gboolean remmina_protocol_widget_start_xport_tunnel(RemminaProtocolWidget *gp, R
 
 	if (!(tunnel = remmina_protocol_widget_init_tunnel(gp))) return FALSE;
 
+	// TRANSLATORS: “%s” is a placeholder for a hostname or IP address.
 	msg = g_strdup_printf(_("Connecting to %s via SSH…"), remmina_file_get_string(gp->priv->remmina_file, "server"));
 	mp = remmina_protocol_widget_mpprogress(gp->cnnobj, msg, cancel_connect_xport_cb, NULL);
 	g_free(msg);
@@ -1067,6 +1303,7 @@ gboolean remmina_protocol_widget_start_xport_tunnel(RemminaProtocolWidget *gp, R
 	tunnel->disconnect_func = remmina_protocol_widget_xport_tunnel_disconnect_callback;
 	tunnel->callback_data = gp;
 
+	REMMINA_DEBUG ("Calling remmina_public_get_server_port");
 	remmina_public_get_server_port(remmina_file_get_string(gp->priv->remmina_file, "server"), 0, &server, NULL);
 	bindlocalhost = (g_strcmp0(REMMINA_SSH(tunnel)->server, server) == 0);
 	g_free(server);
@@ -1108,6 +1345,15 @@ gint remmina_protocol_widget_get_profile_remote_width(RemminaProtocolWidget *gp)
 	return gp->priv->profile_remote_width;
 }
 
+gint remmina_protocol_widget_get_multimon(RemminaProtocolWidget *gp)
+{
+	TRACE_CALL(__func__);
+	/* Returns ehenever multi monitor is enabled (1) */
+	gp->priv->multimon = remmina_file_get_int(gp->priv->remmina_file, "multimon", -1);
+	REMMINA_DEBUG ("Multi monitor is set to %d", gp->priv->multimon);
+	return gp->priv->multimon;
+}
+
 gint remmina_protocol_widget_get_profile_remote_height(RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
@@ -1503,10 +1749,15 @@ gint remmina_protocol_widget_panel_new_certificate(RemminaProtocolWidget *gp, co
 	// For markup see https://developer.gnome.org/pygtk/stable/pango-markup-language.html
 	s = g_strdup_printf(
 		"<big>%s</big>\n\n%s %s\n%s %s\n%s %s\n\n<big>%s</big>",
+		// TRANSLATORS: The user is asked to verify a new SSL certificate.
 		_("Certificate details:"),
+		// TRANSLATORS: An SSL certificate subject is usually the remote server the user connect to.
 		_("Subject:"), subject,
+		// TRANSLATORS: The name or email of the entity that have issued the SSL certificate
 		_("Issuer:"), issuer,
+		// TRANSLATORS: An SSL certificate fingerprint, is a hash of a certificate calculated on all certificate's data and its signature.
 		_("Fingerprint:"), fingerprint,
+		// TRANSLATORS: The user is asked to accept or refuse a new SSL certificate.
 		_("Accept certificate?"));
 	rc = remmina_protocol_widget_dialog(RPWDT_QUESTIONYESNO, gp, 0, s, NULL, NULL, NULL, NULL);
 	g_free(s);
@@ -1530,11 +1781,17 @@ gint remmina_protocol_widget_panel_changed_certificate(RemminaProtocolWidget *gp
 	// For markup see https://developer.gnome.org/pygtk/stable/pango-markup-language.html
 	s = g_strdup_printf(
 		"<big>%s</big>\n\n%s %s\n%s %s\n%s %s\n%s %s\n\n<big>%s</big>",
+		// TRANSLATORS: The user is asked to verify a new SSL certificate.
 		_("The certificate changed! Details:"),
+		// TRANSLATORS: An SSL certificate subject is usually the remote server the user connect to.
 		_("Subject:"), subject,
+		// TRANSLATORS: The name or email of the entity that have issued the SSL certificate
 		_("Issuer:"), issuer,
+		// TRANSLATORS: An SSL certificate fingerprint, is a hash of a certificate calculated on all certificate's data and its signature.
 		_("Old fingerprint:"), old_fingerprint,
+		// TRANSLATORS: An SSL certificate fingerprint, is a hash of a certificate calculated on all certificate's data and its signature.
 		_("New fingerprint:"), new_fingerprint,
+		// TRANSLATORS: The user is asked to accept or refuse a new SSL certificate.
 		_("Accept changed certificate?"));
 	rc = remmina_protocol_widget_dialog(RPWDT_QUESTIONYESNO, gp, 0, s, NULL, NULL, NULL, NULL);
 	g_free(s);
@@ -1677,8 +1934,9 @@ void remmina_protocol_widget_panel_show_listen(RemminaProtocolWidget *gp, gint p
 
 	mp = remmina_message_panel_new();
 	s = g_strdup_printf(
-		_("Listening on port %i for an incoming %s connection…"), port,
-		remmina_file_get_string(gp->priv->remmina_file, "protocol"));
+			// TRANSLATORS: “%i” is a placeholder for a port number. “%s”  is a placeholder for a protocol name (VNC).
+			_("Listening on port %i for an incoming %s connection…"), port,
+			remmina_file_get_string(gp->priv->remmina_file, "protocol"));
 	remmina_message_panel_setup_progress(mp, s, NULL, NULL);
 	g_free(s);
 	gp->priv->listen_message_panel = mp;
@@ -1781,17 +2039,27 @@ void remmina_protocol_widget_setup(RemminaProtocolWidget *gp, RemminaFile *remmi
 									    remmina_file_get_string(remminafile, "protocol"));
 
 	if (!plugin || !plugin->init || !plugin->open_connection) {
+		// TRANSLATORS: “%s” is a placeholder for a protocol name, like “RDP”.
 		remmina_protocol_widget_set_error(gp, _("Install the %s protocol plugin first."),
 						  remmina_file_get_string(remminafile, "protocol"));
 		gp->priv->plugin = NULL;
 		return;
 	}
 	gp->priv->plugin = plugin;
+	gp->plugin = plugin;
 
 	gp->priv->scalemode = remmina_file_get_int(gp->priv->remmina_file, "scale", FALSE);
 	gp->priv->scaler_expand = remmina_file_get_int(gp->priv->remmina_file, "scaler_expand", FALSE);
 }
 
+GtkWindow* remmina_protocol_widget_get_gtkwindow(RemminaProtocolWidget *gp) {
+	return rcw_get_gtkwindow(gp->cnnobj);
+}
+
+GtkWidget *remmina_protocol_widget_gtkviewport(RemminaProtocolWidget *gp) {
+	return rcw_get_gtkviewport(gp->cnnobj);
+}
+
 GtkWidget *remmina_protocol_widget_new(void)
 {
 	return GTK_WIDGET(g_object_new(REMMINA_TYPE_PROTOCOL_WIDGET, NULL));
